var dmtActiveDownloadCount = 0;
window.addEventListener("load", dmtInit, true);

////////////////////////////////////////////////////////////////////////////////
// Opens or closes download manager in the form the user selected (sidebar, tab,
// or window).
function dmtToggleDownloadMgr(open, close)
{
  var ww = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
                     .getService(Components.interfaces.nsIWindowWatcher);

  if (window != ww.activeWindow) return 1;

//  if (window.arguments) alert(window.arguments[0]);

  var preferencesService = Components.classes["@mozilla.org/preferences-service;1"].
                           getService(Components.interfaces.nsIPrefService).
                           getBranch("");
  var winType = preferencesService.getIntPref("extensions.downloadmgr.windowType");

  if (winType == 1)
  {
    // Open Download Manager as a side bar
    if (open && close)
      toggleSidebar('viewDmtSidebar');
    else if (open)
      toggleSidebar('viewDmtSidebar',true);
  }
  else if (winType == 2)
  {
    // Open Download Manager as a tab
    var numTabs = gBrowser.mTabContainer.childNodes.length;
    var index = 0;
    var oldTab = getBrowser().selectedTab;
    var downTab = null;

    // search tabs to see if one of them is the download tab.
    for (index = 0; index < numTabs; index++)
    {
      var tempTab = gBrowser.mTabContainer.childNodes[index];
      var dmtPanel= document.getElementById(tempTab.linkedPanel);

      if (dmtPanel.firstChild.currentURI.spec ==
          'chrome://mozapps/content/downloads/downloads.xul')
      {
        downTab = tempTab;
        break;
      }
    }


    if (downTab)
    {
      // Download tab is already open. Close it if it's the current tab or,
      // make it the current tab if it isn't.
      if (close)
      {
        if (downTab == oldTab)
          BrowserCloseTabOrWindow();
        else
          getBrowser().selectedTab = downTab;
      }
    }
    else
    {
      // No download tab is open, create one.
      var downTab = gBrowser.addTab(
                            'chrome://mozapps/content/downloads/downloads.xul');

      // Setting 
      // When a download is active it might be nice to set:
      //    downTab.image = "chrome://global/skin/throbber/Throbber-small.gif"
      // Conditionally on the preference: "extensions.downloadmgr.tabBusyIcon".
      // Tricky part is handling multiple downloads (can't just start on
      // dl-start and end on dl-done).

      if (!gPrefService.getBoolPref("extensions.downloadmgr.loadTabInBackground"))
        getBrowser().selectedTab = downTab;
      else
        getBrowser().selectedTab = oldTab;
    }
  }
  else
  {
    // Open Download Manager as a window
    if (open)
    {
      toOpenWindowByType('Download:Manager',
                         'chrome://mozapps/content/downloads/downloads.xul',
                         'chrome,dialog=no,resizable');
    }
  }

  return 0;
}

////////////////////////////////////////////////////////////////////////////////
// Called when a shortcut key is used to close the download manager.
function dmtCloseDownloadMgr()
{
  // Try to close Download Manager in a side bar
  var sidebarWindow = document.getElementById("sidebar").contentWindow;
  if (sidebarWindow.location.href ==
      "chrome://mozapps/content/downloads/downloads.xul")
  {
    toggleSidebar('viewDmtSidebar', false);
    return;
  }

  // Not in a sidebar, try to close Download Manager in a tab
  // Check if current tab is the DM
  if (getWebNavigation().currentURI.spec ==
      "chrome://mozapps/content/downloads/downloads.xul")
  {
    BrowserCloseTabOrWindow();
    return;
  }

  // Not in a sidebar or tab, close Download Manager in a separate window
  dmtWindowClose(0);
}

////////////////////////////////////////////////////////////////////////////////
// Handles cases when download manager is actually in a window.
function dmtWindowClose(count)
{
  if (count > 5) return;
  var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].
                      getService();
  var windowManagerInterface = windowManager.QueryInterface(
                                       Components.interfaces.nsIWindowMediator);
  var windowObj = windowManagerInterface.getMostRecentWindow("Download:Manager");

  if (!windowObj)
  {
    window.setTimeout(dmtWindowClose, 100, count + 1);
  }
  else
  {
    windowObj.close();
  }
}

////////////////////////////////////////////////////////////////////////////////
// Automatically close the download manager when downloads have completed.
// This closes all of them, if multiple are open.
function dmtAutoClose()
{
  // Close Download Manager in a side bar
  var sidebarWindow = document.getElementById("sidebar").contentWindow;
  if (sidebarWindow.location.href ==
      "chrome://mozapps/content/downloads/downloads.xul")
  {
    toggleSidebar('viewDmtSidebar', false);
  }

  // Close Download Manager in a tab
  {
    var numTabs = gBrowser.mTabContainer.childNodes.length;
    var index = 0;
    var oldTab = getBrowser().selectedTab;

    if (getWebNavigation().currentURI.spec ==
        "chrome://mozapps/content/downloads/downloads.xul")
      oldTab = null;

    // Find and close the DM tab (if one is open)
    for (index = numTabs - 1; index >= 0; index--)
    {
      var tempTab = gBrowser.mTabContainer.childNodes[index];
      var dmtPanel = document.getElementById(tempTab.linkedPanel);

      if (dmtPanel.firstChild.currentURI.spec ==
          "chrome://mozapps/content/downloads/downloads.xul")
      {
        // Found a DMT tab, close it.
        getBrowser().selectedTab = tempTab;
        BrowserCloseTabOrWindow();
      }
    }

    // Restore selected tab if it wasn't DMT. If it was DMT then just let
    // the browser decide which window to select.
    if (oldTab != null)
      getBrowser().selectedTab = oldTab;
  }

  // Close Download Manager in a separate window
  dmtWindowClose(0);
}

////////////////////////////////////////////////////////////////////////////////
// When download window is opened there are several complications.
// 1) The user may have selected to open after a delay. This can be scheduled
//    for that to happen.
// 2) The user may have selected when download is complete and download happened
//    so fast that it beat the window opening (especially is open is after a
//    delay). In this case don't open the window at all.
// 3) The open in dmtToggleDownloadMgr() may fail because it is called before
//    the window has been activated. It is recalled after a delay to fix this.
function dmtDelayedOpen(first)
{
  if (first)
  {
    // This was called from dmtDownloadObserver. Don't show if download finished
    // before the user specified delay.
    if (dmtActiveDownloadCount > 0)
    {
      if (dmtToggleDownloadMgr(true, false))
        window.setTimeout(dmtDelayedOpen, 500, false);
    }
  }
  else
  {
    // This was a recursive call. If a download is active then show no matter
    // what, otherwise more work is needed.
    if (dmtActiveDownloadCount > 0)
    {
      dmtToggleDownloadMgr(true, false);
    }
    else
    {
      // The download finished between the first call and this recursive call.
      // If set to close when download completes then don't show at all, but
      // if not then show it even if no download is active, because it finished
      // after the user specified delay.
      try {
        var pref = Components.classes["@mozilla.org/preferences-service;1"].
                   getService(Components.interfaces.nsIPrefBranch);
        var close = pref.getBoolPref("extensions.downloadmgr.closeWhenDone");
      } catch (e) {close = false;}
      if (!close)
        dmtToggleDownloadMgr(true, false);
    }
  }
}

////////////////////////////////////////////////////////////////////////////////
// Class to handle notifications from nsIObserverService
var dmtDownloadObserver = {
  observe: function (aSubject, aTopic, aState)
  {
    var pref = Components.classes["@mozilla.org/preferences-service;1"].
               getService(Components.interfaces.nsIPrefBranch);
    switch (aTopic) {
    case "dl-start":
      dmtActiveDownloadCount = dmtActiveDownloadCount + 1;
      try {
        var show = pref.getBoolPref("extensions.downloadmgr.showWhenStarting");
        var delay = pref.getIntPref("browser.download.manager.openDelay");
      } catch (e) {show = false;}
      if (show)
      {
        if (delay == 0)
          dmtDelayedOpen(true);
        else
          window.setTimeout(dmtDelayedOpen, delay, true);
      }
      break;

    case "dl-failed":
    case "dl-cancel":
      if (dmtActiveDownloadCount > 0)
        dmtActiveDownloadCount = dmtActiveDownloadCount - 1;
      break;

    case "dl-done":
      if (dmtActiveDownloadCount > 0)
        dmtActiveDownloadCount = dmtActiveDownloadCount - 1;
      try {
        var close = pref.getBoolPref("extensions.downloadmgr.closeWhenDone");
      } catch (e) {close = false;}
      if (close && (dmtActiveDownloadCount == 0))
        dmtAutoClose();
      break;

    case "DMT-ui-close":
      dmtCloseDownloadMgr();
      break;
    }
  }
};

////////////////////////////////////////////////////////////////////////////////
// Called at Firefox startup. Sets up observers other one time initialization.
function dmtInit()
{
  // Only ever call once
  window.removeEventListener("load", dmtInit, true);

  // Handlers for events generated by the Download Manager (download events)
  var observerService = Components.classes["@mozilla.org/observer-service;1"].
                        getService(Components.interfaces.nsIObserverService);
  observerService.addObserver(dmtDownloadObserver, "dl-start",     false);
  observerService.addObserver(dmtDownloadObserver, "dl-failed",    false);
  observerService.addObserver(dmtDownloadObserver, "dl-cancel",    false);
  observerService.addObserver(dmtDownloadObserver, "dl-done",      false);
  observerService.addObserver(dmtDownloadObserver, "DMT-ui-close", false);

  // Force some existing preferences off. DMT has it's own, which handle tab and
  // sidebar as well as the window.
  var pref = Components.classes["@mozilla.org/preferences-service;1"].
             getService(Components.interfaces.nsIPrefBranch);
  try {
    pref.setBoolPref("browser.download.manager.showWhenStarting", false);
    pref.setBoolPref("browser.download.manager.closeWhenDone", false);
  } catch (e) {}
}

////////////////////////////////////////////////////////////////////////////////
// Cleans up observers when Firefox exits. Does not appear to be called.
// Pointless to cleanup if Firefox is exiting anyway.
function dmtExit(aEvent)
{
  var observerService = Components.classes["@mozilla.org/observer-service;1"].
                        getService(Components.interfaces.nsIObserverService);

  observerService.removeObserver(dmtDownloadObserver, "DMT-ui-close");
  observerService.removeObserver(dmtDownloadObserver, "dl-done");
  observerService.removeObserver(dmtDownloadObserver, "dl-cancel");
  observerService.removeObserver(dmtDownloadObserver, "dl-failed");
  observerService.removeObserver(dmtDownloadObserver, "dl-start");
}

